//
// Copyright (c) Microsoft.  All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using Microsoft.Rest.Azure;
using Microsoft.Azure.Management.Resources;
using Microsoft.Azure.Management.Resources.Models;
using Microsoft.Azure.Test;
using Xunit;
using Microsoft.Rest.ClientRuntime.Azure.TestFramework;

namespace ResourceGroups.Tests
{
    public class LiveProviderTests : TestBase
    {
        private const string ProviderName = "microsoft.insights";
        public ResourceManagementClient GetResourceManagementClient(MockContext context, RecordedDelegatingHandler handler)
        {
            handler.IsPassThrough = true;
            return this.GetResourceManagementClientWithHandler(context, handler);
        }

        [Fact]
        public void ProviderGetValidateMessage()
        {
            using (MockContext context = MockContext.Start(this.GetType().FullName))
            {
                var handler = new RecordedDelegatingHandler() { StatusCodeToReturn = HttpStatusCode.OK };

                var client = GetResourceManagementClient(context, handler);

                var reg = client.Providers.Register(ProviderName);
                Assert.NotNull(reg);

                var result = client.Providers.Get(ProviderName);

                // Validate headers
                Assert.Equal(HttpMethod.Get, handler.Method);
                Assert.NotNull(handler.RequestHeaders.GetValues("Authorization"));

                // Validate result
                Assert.NotNull(result);
                Assert.NotEmpty(result.Id);
                Assert.Equal(ProviderName, result.NamespaceProperty);
                Assert.True("Registered" == result.RegistrationState ||
                    "Registering" == result.RegistrationState,
                    string.Format("Provider registration state was not 'Registered' or 'Registering', instead it was '{0}'", result.RegistrationState));
                Assert.NotEmpty(result.ResourceTypes);
                Assert.NotEmpty(result.ResourceTypes[0].Locations);
            }
        }

        [Fact]
        public void ProviderListValidateMessage()
        {
            using (MockContext context = MockContext.Start(this.GetType().FullName))
            {
                var handler = new RecordedDelegatingHandler { StatusCodeToReturn = HttpStatusCode.OK };

                var client = GetResourceManagementClient(context, handler);

                var reg = client.Providers.Register(ProviderName);
                Assert.NotNull(reg);

                var result = client.Providers.List(null);

                // Validate headers
                Assert.Equal(HttpMethod.Get, handler.Method);
                Assert.NotNull(handler.RequestHeaders.GetValues("Authorization"));

                // Validate result
                Assert.True(result.Any());
                var websiteProvider =
                    result.First(
                        p => p.NamespaceProperty.Equals(ProviderName, StringComparison.OrdinalIgnoreCase));
                Assert.Equal(ProviderName, websiteProvider.NamespaceProperty);
                Assert.True("Registered" == websiteProvider.RegistrationState ||
                    "Registering" == websiteProvider.RegistrationState,
                    string.Format("Provider registration state was not 'Registered' or 'Registering', instead it was '{0}'", websiteProvider.RegistrationState));
                Assert.NotEmpty(websiteProvider.ResourceTypes);
                Assert.NotEmpty(websiteProvider.ResourceTypes[0].Locations);
            }
        }

        [Fact]
        public void VerifyProviderRegister()
        {
            var handler = new RecordedDelegatingHandler() {StatusCodeToReturn = HttpStatusCode.OK};
            using (MockContext context = MockContext.Start(this.GetType().FullName))
            {
                var client = GetResourceManagementClient(context, handler);

                client.Providers.Register(ProviderName);

                var provider = client.Providers.Get(ProviderName);
                Assert.True(provider.RegistrationState == "Registered" ||
                            provider.RegistrationState == "Registering");
            }
        }

        [Fact]
        public void VerifyProviderUnregister()
        {
            var handler = new RecordedDelegatingHandler() { StatusCodeToReturn = HttpStatusCode.OK };
            using (MockContext context = MockContext.Start(this.GetType().FullName))
            {
                var client = GetResourceManagementClient(context, handler);

                var registerResult = client.Providers.Register(ProviderName);

                var provider = client.Providers.Get(ProviderName);
                Assert.True(provider.RegistrationState == "Registered" ||
                            provider.RegistrationState == "Registering");

                var unregisterResult = client.Providers.Unregister(ProviderName);

                provider = client.Providers.Get(ProviderName);
                Assert.True(provider.RegistrationState == "NotRegistered" ||
                            provider.RegistrationState == "Unregistering",
                            "RegistrationState is expected NotRegistered or Unregistering. Actual value " +
                            provider.RegistrationState);
            }
        }

        [Fact]
        public void ProviderOperationsList()
        {
            using (MockContext context = MockContext.Start(this.GetType().FullName))
            {
                const string DefaultApiVersion = "2014-06-01";

                var handler = new RecordedDelegatingHandler { StatusCodeToReturn = HttpStatusCode.OK };

                var client = GetResourceManagementClient(context, handler);
                var insightsProvider = client.Providers.Get(ProviderName);

                // Validate result
                Assert.True(insightsProvider != null);

                // Validate headers
                Assert.Equal(HttpMethod.Get, handler.Method);
                Assert.NotNull(handler.RequestHeaders.GetValues("Authorization"));

                Assert.NotEmpty(insightsProvider.ResourceTypes);
                var operationResourceType = insightsProvider.ResourceTypes.Single(x => x.ResourceType == "operations");
                IList<string> operationsSupportedApiVersions = operationResourceType.ApiVersions;
                string latestSupportedApiVersion = DefaultApiVersion;

                if (operationsSupportedApiVersions != null && operationsSupportedApiVersions.Any())
                {
                    latestSupportedApiVersion = operationsSupportedApiVersions.First();
                }

                ResourceIdentity identity = new ResourceIdentity
                {
                    ResourceName = string.Empty,
                    ResourceType = "operations",
                    ResourceProviderNamespace = ProviderName,
                    ResourceProviderApiVersion = latestSupportedApiVersion
                };

                var operations = client.ResourceProviderOperationDetails.List(identity.ResourceProviderNamespace, identity.ResourceProviderApiVersion);

                Assert.NotEmpty(operations);
                Assert.NotEmpty(operations.First().Name);
                Assert.NotNull(operations.First().Display);
                IEnumerable<ResourceProviderOperationDefinition> definitions =
                    operations.Where(op => string.Equals(op.Name, "Microsoft.Insights/AlertRules/Write", StringComparison.OrdinalIgnoreCase));
                Assert.NotNull(definitions);
                Assert.NotEmpty(definitions);
                Assert.Equal(1, definitions.Count());

                // Negative case with unsupported api version
                identity = new ResourceIdentity
                {
                    ResourceName = string.Empty,
                    ResourceType = "operations",
                    ResourceProviderNamespace = ProviderName,
                    ResourceProviderApiVersion = "2015-01-01"
                };

                Assert.Throws<CloudException>(() => client.ResourceProviderOperationDetails.List(identity.ResourceProviderNamespace, identity.ResourceProviderApiVersion));
            }
        }
    }
}
